"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Draggable = Draggable;
exports.useDraggable = useDraggable;

var _react = require("react");

var _propTypes = _interopRequireDefault(require("prop-types"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable id-length */
function Draggable({
  children,
  ...rest
}) {
  return children(useDraggable(rest));
}

Draggable.propTypes = {
  children: _propTypes.default.func.isRequired,
  controlStyle: _propTypes.default.bool
};

function useDraggable({
  controlStyle
} = {}) {
  const targetRef = (0, _react.useRef)(null);
  const handleRef = (0, _react.useRef)(null);
  const [dragging, setDragging] = (0, _react.useState)(null);
  const [prev, setPrev] = (0, _react.useState)({
    x: 0,
    y: 0
  });
  const [delta, setDelta] = (0, _react.useState)({
    x: 0,
    y: 0
  });
  const initial = (0, _react.useRef)({
    x: 0,
    y: 0
  });
  (0, _react.useEffect)(() => {
    const handle = handleRef.current || targetRef.current;
    handle.addEventListener('mousedown', startDragging);
    handle.addEventListener('touchstart', startDragging);
    return () => {
      handle.removeEventListener('mousedown', startDragging);
      handle.removeEventListener('touchstart', startDragging);
    };

    function startDragging(event) {
      setDragging(true);
      const source = event.touches && event.touches[0] || event;
      const {
        clientX,
        clientY
      } = source;
      initial.current = {
        x: clientX,
        y: clientY
      };
    }
  }, [controlStyle]);
  (0, _react.useEffect)(() => {
    const handle = handleRef.current || targetRef.current;

    if (dragging) {
      document.addEventListener('mousemove', reposition, {
        passive: true
      });
      document.addEventListener('touchmove', reposition, {
        passive: true
      });
      document.addEventListener('mouseup', stopDragging);
      document.addEventListener('touchend', stopDragging);
    } else {
      document.removeEventListener('mousemove', reposition, {
        passive: true
      });
      document.removeEventListener('mouseup', stopDragging);
      document.removeEventListener('touchmove', reposition, {
        passive: true
      });
      document.removeEventListener('touchend', stopDragging);
    }

    if (controlStyle) {
      handle.style.cursor = dragging ? 'grabbing' : 'grab';
    }

    return () => {
      if (controlStyle) {
        handle.style.cursor = '';
      }

      document.removeEventListener('mousemove', reposition, {
        passive: true
      });
      document.removeEventListener('mouseup', stopDragging);
      document.removeEventListener('touchmove', reposition, {
        passive: true
      });
      document.removeEventListener('touchend', stopDragging);
    };

    function stopDragging(event) {
      setDragging(false);
      reposition(event, true);
    }

    function reposition(event, finished) {
      const source = event.changedTouches && event.changedTouches[0] || event.touches && event.touches[0] || event;
      const {
        clientX,
        clientY
      } = source;
      const calculatedX = clientX - initial.current.x;
      const calculatedY = clientY - initial.current.y;
      const newDelta = {
        x: calculatedX + prev.x,
        y: calculatedY + prev.y
      };
      setDelta(newDelta);

      if (finished) {
        setPrev(newDelta);
      }
    }
  }, [dragging, prev, controlStyle]);
  (0, _react.useEffect)(() => {
    if (controlStyle) {
      targetRef.current.style.transform = `translate(${delta.x}px, ${delta.y}px)`;
    }
  }, [delta, controlStyle]);

  const getTargetProps = () => ({
    'aria-grabbed': dragging || null
  });

  return {
    targetRef,
    handleRef,
    getTargetProps,
    dragging,
    delta
  };
}