"use strict";

var _react = _interopRequireDefault(require("react"));

var _chai = require("chai");

var _react2 = require("@testing-library/react");

var _Draggable = require("./Draggable");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

describe('draggable', () => {
  let utils;
  beforeEach(() => {
    utils = setup();
  });
  afterEach(() => {
    (0, _react2.cleanup)();
  });
  it('should supply proper props to target', () => {
    (0, _chai.expect)(utils.container.querySelector('[aria-grabbed]')).not.to.be.ok;
  });
  describe('starting drag', () => {
    it('should supply proper props to target', () => {
      const {
        getByText,
        container
      } = utils;

      _react2.fireEvent.mouseDown(getByText(/handle/));

      (0, _chai.expect)(container.querySelector('[aria-grabbed]')).to.be.ok;
    });
    it('should return a delta position', () => {
      const startAt = {
        clientX: 10,
        clientY: 10
      };
      const delta = {
        x: 5,
        y: 5
      };
      const {
        drag,
        getByText
      } = utils;
      drag({
        start: startAt,
        delta
      });
      (0, _chai.expect)(getByText(`${delta.x}, ${delta.y}`)).to.be.ok;
    });
    it('should return a correct delta position after second drag', () => {
      const startAt = {
        clientX: 10,
        clientY: 10
      };
      const delta = {
        x: 5,
        y: 5
      };
      const secondStart = {
        clientX: startAt.clientX + delta.x,
        clientY: startAt.clientY + delta.y
      };
      const {
        drag,
        getByText
      } = utils;
      drag({
        start: startAt,
        delta
      });
      drag({
        start: secondStart,
        delta
      });
      (0, _chai.expect)(getByText(`${2 * delta.x}, ${2 * delta.y}`)).to.be.ok;
    });
    describe('decorate with styles', () => {
      beforeEach(() => {
        (0, _react2.cleanup)();
        utils = setup({
          controlStyle: true
        });
      });
      it("should change target's style when dragging", () => {
        const {
          getByTestId,
          drag
        } = utils;
        drag({
          start: {
            clientX: 3,
            clientY: 3
          },
          delta: {
            x: 10,
            y: 15
          }
        });
        (0, _chai.expect)(getByTestId('main').style.transform).to.equal(`translate(10px, 15px)`);
      });
      it("should change target's style when dragging (touches)", () => {
        const {
          getByTestId,
          drag
        } = utils;
        drag({
          start: {
            clientX: 3,
            clientY: 3
          },
          delta: {
            x: 10,
            y: 15
          },
          touch: true
        });
        (0, _chai.expect)(getByTestId('main').style.transform).to.equal(`translate(10px, 15px)`);
      });
      it('should set proper cursor', () => {
        (0, _chai.expect)(utils.getByText(/handle/i).style.cursor).to.equal('grab');
      });
      it('should change cursor while dragging', () => {
        const {
          getByText
        } = utils;

        _react2.fireEvent.mouseDown(getByText(/handle/i));

        (0, _chai.expect)(getByText(/handle/i).style.cursor).to.equal('grabbing');
      });
    });
    describe('ending drag', () => {
      it('should supply proper props to target', () => {
        const {
          drag,
          container
        } = utils;
        drag();
        (0, _chai.expect)(container.querySelector('[aria-grabbed]')).not.to.be.ok;
      });
    });
  });

  function Consumer(props) {
    const {
      targetRef,
      handleRef,
      getTargetProps,
      delta,
      dragging
    } = (0, _Draggable.useDraggable)(props);
    return _react.default.createElement("main", _extends({
      className: "container",
      ref: targetRef,
      "data-testid": "main",
      style: {
        position: 'fixed',
        top: '11px',
        left: '11px'
      }
    }, getTargetProps()), dragging && _react.default.createElement("span", null, "Dragging to:"), _react.default.createElement("output", null, delta.x, ", ", delta.y), _react.default.createElement("button", {
      className: "handle",
      ref: handleRef
    }, "handle"));
  }

  function setup(props) {
    const getters = (0, _react2.render)(_react.default.createElement(Consumer, props));

    function drag({
      start = {
        clientX: 0,
        clientY: 0
      },
      delta = {
        x: 0,
        y: 0
      },
      touch = false
    } = {}) {
      if (touch) {
        _react2.fireEvent.touchStart(getters.getByText(/handle/), {
          touches: [start]
        });

        _react2.fireEvent.touchMove(getters.getByText(/handle/), {
          touches: [{
            clientX: start.clientX + delta.x,
            clientY: start.clientY + delta.y
          }]
        });

        _react2.fireEvent.touchEnd(getters.getByText(/handle/), {
          changedTouches: [{
            clientX: start.clientX + delta.x,
            clientY: start.clientY + delta.y
          }]
        });
      } else {
        _react2.fireEvent.mouseDown(getters.getByText(/handle/), start);

        _react2.fireEvent.mouseMove(getters.getByText(/handle/), {
          clientX: start.clientX + delta.x,
          clientY: start.clientY + delta.y
        });

        _react2.fireEvent.mouseUp(getters.getByText(/handle/), {
          clientX: start.clientX + delta.x,
          clientY: start.clientY + delta.y
        });
      }
    }

    return { ...getters,
      drag
    };
  }
});